@@ -124,7 +124,7 @@ struct mk_sched_conn *mk_server_listen_handler(struct mk_sched_worker *sched,
 
 error:
     if (client_fd != -1) {
-        listener->network->network->close(client_fd);
+        listener->network->network->close(listener->network, client_fd);
     }
 
     return NULL;
@@ -157,7 +157,7 @@ void mk_server_listen_exit(struct mk_list *list)
 
     mk_list_foreach_safe(head, tmp, list) {
         listen = mk_list_entry(head, struct mk_server_listen, _head);
-        close(listen->server_fd);
+        mk_event_closesocket(listen->server_fd);
         mk_list_del(&listen->_head);
         mk_mem_free(listen);
     }
@@ -167,7 +167,6 @@ void mk_server_listen_exit(struct mk_list *list)
 
 struct mk_list *mk_server_listen_init(struct mk_server *server)
 {
-    int i = 0;
     int server_fd;
     int reuse_port = MK_FALSE;
     struct mk_list *head;
@@ -203,7 +202,7 @@ struct mk_list *mk_server_listen_init(struct mk_server *server)
 #endif
             }
 
-            listener = mk_mem_alloc(sizeof(struct mk_server_listen));
+            listener = mk_mem_alloc_z(sizeof(struct mk_server_listen));
 
             /* configure the internal event_state */
             event = &listener->event;
@@ -254,7 +253,6 @@ struct mk_list *mk_server_listen_init(struct mk_server *server)
                    listen->port);
             return NULL;
         }
-        i += 1;
     }
 
     if (reuse_port == MK_TRUE) {
@@ -318,12 +316,16 @@ static int mk_server_fifo_worker_setup(struct mk_event_loop *evl)
  */
 void mk_server_loop_balancer(struct mk_server *server)
 {
+    size_t bytes;
+    uint64_t val;
+    int operation_flag;
     struct mk_list *head;
     struct mk_list *listeners;
     struct mk_server_listen *listener;
     struct mk_event *event;
     struct mk_event_loop *evl;
     struct mk_sched_worker *sched;
+    struct mk_event management_event;
 
     /* Init the listeners */
     listeners = mk_server_listen_init(server);
@@ -347,10 +349,42 @@ void mk_server_loop_balancer(struct mk_server *server)
                      listener);
     }
 
-    while (1) {
+    memset(&management_event, 0, sizeof(struct mk_event));
+
+    mk_event_add(evl,
+                 server->lib_ch_manager[0],
+                 MK_EVENT_NOTIFICATION,
+                 MK_EVENT_READ,
+                 &management_event);
+
+    operation_flag = MK_TRUE;
+    while (operation_flag) {
         mk_event_wait(evl);
         mk_event_foreach(event, evl) {
             if (event->mask & MK_EVENT_READ) {
+                /* This signal is sent by mk_stop and both this and
+                 * mk_lib_worker are expecting it.
+                 */
+                if (server->lib_ch_manager[0] == event->fd) {
+#ifdef _WIN32
+        bytes = recv(event->fd, &val, sizeof(uint64_t), MSG_WAITALL);
+#else
+        bytes = read(event->fd, &val, sizeof(uint64_t));
+#endif
+
+                    if (bytes <= 0) {
+                        return;
+                    }
+
+                    if (val == MK_SERVER_SIGNAL_STOP) {
+                        operation_flag = MK_FALSE;
+
+                        break;
+                    }
+
+                    continue;
+                }
+
                 /*
                  * Accept connection: determinate which thread may work on this
                  * new connection.
@@ -384,6 +418,8 @@ void mk_server_loop_balancer(struct mk_server *server)
             }
         }
     }
+    mk_event_loop_destroy(evl);
+    mk_server_listen_exit(listeners);
 }
 
 /*
@@ -462,7 +498,7 @@ void mk_server_worker_loop(struct mk_server *server)
     }
 
     /* create a new timeout file descriptor */
-    server_timeout = mk_mem_alloc(sizeof(struct mk_server_timeout));
+    server_timeout = mk_mem_alloc_z(sizeof(struct mk_server_timeout));
     MK_TLS_SET(mk_tls_server_timeout, server_timeout);
     timeout_fd = mk_event_timeout_create(evl, server->timeout, 0, server_timeout);
 
@@ -540,7 +576,7 @@ void mk_server_worker_loop(struct mk_server *server)
                     }
                     else if (val == MK_SCHED_SIGNAL_FREE_ALL) {
                         if (timeout_fd > 0) {
-                            close(timeout_fd);
+                            mk_event_timeout_destroy(evl, server_timeout);
                         }
                         mk_mem_free(MK_TLS_GET(mk_tls_server_timeout));
                         mk_server_listen_exit(sched->listeners);
@@ -598,20 +634,19 @@ static int mk_server_lib_notify_started(struct mk_server *server)
     uint64_t val;
 
     /* Check the channel is valid (enabled by library mode) */
-    if (server->lib_ch_manager[1] <= 0) {
+    if (server->lib_ch_start[1] <= 0) {
         return -1;
     }
 
     val = MK_SERVER_SIGNAL_START;
 
 #ifdef _WIN32
-    return send(server->lib_ch_manager[1], &val, sizeof(uint64_t), 0);
+    return send(server->lib_ch_start[1], &val, sizeof(uint64_t), 0);
 #else
-    return write(server->lib_ch_manager[1], &val, sizeof(uint64_t));
+    return write(server->lib_ch_start[1], &val, sizeof(uint64_t));
 #endif
 }
 
-
 void mk_server_loop(struct mk_server *server)
 {
     uint64_t val;
