@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,10 +21,12 @@
 #define FLB_FILTER_EXPECT_H
 
 #include <fluent-bit/flb_filter_plugin.h>
+#include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_record_accessor.h>
 
 #define FLB_EXP_WARN              0
 #define FLB_EXP_EXIT              1
+#define FLB_EXP_RESULT_KEY        2
 
 /* Rule types */
 #define FLB_EXP_KEY_EXISTS        0   /* key exists */
@@ -44,6 +45,7 @@ struct flb_expect_rule {
 
 struct flb_expect {
     int action;
+    flb_sds_t result_key;
     struct mk_list rules;
     struct flb_filter_instance *ins;
 };
