@@ -2,8 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -27,6 +26,8 @@
 #include <fluent-bit/flb_pack.h>
 #include <fluent-bit/flb_log.h>
 #include <fluent-bit/flb_time.h>
+#include <fluent-bit/flb_log_event_decoder.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 #include <msgpack.h>
 #include "stdlib.h"
 
@@ -36,6 +37,8 @@
 #include <stdio.h>
 #include <sys/types.h>
 
+pthread_mutex_t throttle_mut;
+
 
 static bool apply_suffix (double *x, char suffix_char)
 {
@@ -68,30 +71,30 @@ static bool apply_suffix (double *x, char suffix_char)
 
 void *time_ticker(void *args)
 {
-    struct ticker *t = args;
     struct flb_time ftm;
     long timestamp;
-    struct flb_filter_throttle_ctx *ctx = t->ctx;
+    struct flb_filter_throttle_ctx *ctx = args;
 
-    while (!t->done) {
+    while (1) {
         flb_time_get(&ftm);
         timestamp = flb_time_to_double(&ftm);
-        window_add(t->ctx->hash, timestamp, 0);
+        pthread_mutex_lock(&throttle_mut);
+        window_add(ctx->hash, timestamp, 0);
 
-        t->ctx->hash->current_timestamp = timestamp;
+        ctx->hash->current_timestamp = timestamp;
 
-        if (t->ctx->print_status) {
+        if (ctx->print_status) {
             flb_plg_info(ctx->ins,
                          "%ld: limit is %0.2f per %s with window size of %i, "
                          "current rate is: %i per interval",
-                         timestamp, t->ctx->max_rate, t->ctx->slide_interval,
-                         t->ctx->window_size,
-                         t->ctx->hash->total / t->ctx->hash->size);
+                         timestamp, ctx->max_rate, ctx->slide_interval,
+                         ctx->window_size,
+                         ctx->hash->total / ctx->hash->size);
         }
-        sleep(t->seconds);
+        pthread_mutex_unlock(&throttle_mut);
+        /* sleep is a cancelable function */
+        sleep(ctx->ticker_data.seconds);
     }
-
-    return NULL;
 }
 
 /* Given a msgpack record, do some filter action based on the defined rules */
@@ -108,42 +111,20 @@ static inline int throttle_data(struct flb_filter_throttle_ctx *ctx)
 
 static int configure(struct flb_filter_throttle_ctx *ctx, struct flb_filter_instance *f_ins)
 {
-    const char *str = NULL;
-    double val  = 0;
-    char *endp;
+    int ret;
 
-    /* rate per second */
-    str = flb_filter_get_property("rate", f_ins);
-
-    if (str != NULL && (val = strtod(str, &endp)) > 1) {
-        ctx->max_rate = val;
-    } else {
-        ctx->max_rate = THROTTLE_DEFAULT_RATE;
+    ret = flb_filter_config_map_set(f_ins, ctx);
+    if (ret == -1)  {
+        flb_plg_error(f_ins, "unable to load configuration");
+        return -1;
+    }
+    if (ctx->max_rate <= 1.0) {
+        ctx->max_rate = strtod(THROTTLE_DEFAULT_RATE, NULL);
+    }
+    if (ctx->window_size <= 1) {
+        ctx->window_size = strtoul(THROTTLE_DEFAULT_WINDOW, NULL, 10);
     }
 
-    /* windows size */
-    str = flb_filter_get_property("window", f_ins);
-    if (str != NULL && (val = strtoul(str, &endp, 10)) > 1) {
-        ctx->window_size = val;
-    } else {
-        ctx->window_size = THROTTLE_DEFAULT_WINDOW;
-    }
-
-    /* print informational status */
-    str = flb_filter_get_property("print_status", f_ins);
-    if (str != NULL) {
-        ctx->print_status = flb_utils_bool(str);
-    } else {
-        ctx->print_status = THROTTLE_DEFAULT_STATUS;
-    }
-
-    /* sliding interval */
-    str = flb_filter_get_property("interval", f_ins);
-    if (str != NULL) {
-        ctx->slide_interval = str;
-    } else {
-        ctx->slide_interval = THROTTLE_DEFAULT_INTERVAL;
-    }
     return 0;
 }
 
@@ -176,12 +157,12 @@ static int cb_throttle_init(struct flb_filter_instance *f_ins,
                         void *data)
 {
     int ret;
-    pthread_t tid;
-    struct ticker *ticker_ctx;
     struct flb_filter_throttle_ctx *ctx;
 
+    pthread_mutex_init(&throttle_mut, NULL);
+
     /* Create context */
-    ctx = flb_malloc(sizeof(struct flb_filter_throttle_ctx));
+    ctx = flb_calloc(1, sizeof(struct flb_filter_throttle_ctx));
     if (!ctx) {
         flb_errno();
         return -1;
@@ -195,22 +176,13 @@ static int cb_throttle_init(struct flb_filter_instance *f_ins,
         return -1;
     }
 
-    ticker_ctx = flb_malloc(sizeof(struct ticker));
-    if (!ticker_ctx) {
-        flb_errno();
-        flb_free(ctx);
-        return -1;
-    }
-
     /* Set our context */
     flb_filter_set_context(f_ins, ctx);
 
     ctx->hash = window_create(ctx->window_size);
 
-    ticker_ctx->ctx = ctx;
-    ticker_ctx->done = false;
-    ticker_ctx->seconds = parse_duration(ctx, ctx->slide_interval);
-    pthread_create(&tid, NULL, &time_ticker, ticker_ctx);
+    ctx->ticker_data.seconds = parse_duration(ctx, ctx->slide_interval);
+    pthread_create(&ctx->ticker_data.thr, NULL, &time_ticker, ctx);
     return 0;
 }
 
@@ -225,70 +197,141 @@ static int cb_throttle_filter(const void *data, size_t bytes,
     int ret;
     int old_size = 0;
     int new_size = 0;
-    msgpack_unpacked result;
-    msgpack_object root;
-    size_t off = 0;
+    struct flb_log_event_encoder log_encoder;
+    struct flb_log_event_decoder log_decoder;
+    struct flb_log_event log_event;
+
     (void) f_ins;
     (void) i_ins;
     (void) config;
-    msgpack_sbuffer tmp_sbuf;
-    msgpack_packer tmp_pck;
 
-    /* Create temporary msgpack buffer */
-    msgpack_sbuffer_init(&tmp_sbuf);
-    msgpack_packer_init(&tmp_pck, &tmp_sbuf, msgpack_sbuffer_write);
+    ret = flb_log_event_decoder_init(&log_decoder, (char *) data, bytes);
 
+    if (ret != FLB_EVENT_DECODER_SUCCESS) {
+        flb_plg_error(f_ins,
+                      "Log event decoder initialization error : %d", ret);
+
+        return FLB_FILTER_NOTOUCH;
+    }
+
+    ret = flb_log_event_encoder_init(&log_encoder,
+                                     FLB_LOG_EVENT_FORMAT_DEFAULT);
+
+    if (ret != FLB_EVENT_ENCODER_SUCCESS) {
+        flb_plg_error(f_ins,
+                      "Log event encoder initialization error : %d", ret);
+
+        flb_log_event_decoder_destroy(&log_decoder);
+
+        return FLB_FILTER_NOTOUCH;
+    }
 
     /* Iterate each item array and apply rules */
-    msgpack_unpacked_init(&result);
-    while (msgpack_unpack_next(&result, data, bytes, &off) == MSGPACK_UNPACK_SUCCESS) {
-        root = result.data;
-        if (root.type != MSGPACK_OBJECT_ARRAY) {
-            continue;
-        }
-
+    while ((ret = flb_log_event_decoder_next(
+                    &log_decoder,
+                    &log_event)) == FLB_EVENT_DECODER_SUCCESS) {
         old_size++;
-
+        pthread_mutex_lock(&throttle_mut);
         ret = throttle_data(context);
+        pthread_mutex_unlock(&throttle_mut);
+
         if (ret == THROTTLE_RET_KEEP) {
-            msgpack_pack_object(&tmp_pck, root);
-            new_size++;
+            ret = flb_log_event_encoder_emit_raw_record(
+                        &log_encoder,
+                        &((char *) data)[log_decoder.previous_offset],
+                        log_decoder.record_length);
+
+            if (ret == FLB_EVENT_ENCODER_SUCCESS) {
+                new_size++;
+            }
         }
         else if (ret == THROTTLE_RET_DROP) {
             /* Do nothing */
         }
     }
-    msgpack_unpacked_destroy(&result);
 
     /* we keep everything ? */
     if (old_size == new_size) {
         /* Destroy the buffer to avoid more overhead */
-        msgpack_sbuffer_destroy(&tmp_sbuf);
-        return FLB_FILTER_NOTOUCH;
+        ret = FLB_FILTER_NOTOUCH;
+    }
+    else {
+        *out_buf  = log_encoder.output_buffer;
+        *out_size = log_encoder.output_length;
+
+        flb_log_event_encoder_claim_internal_buffer_ownership(&log_encoder);
+
+        ret = FLB_FILTER_MODIFIED;
     }
 
-    /* link new buffers */
-    *out_buf   = tmp_sbuf.data;
-    *out_size = tmp_sbuf.size;
+    flb_log_event_decoder_destroy(&log_decoder);
+    flb_log_event_encoder_destroy(&log_encoder);
 
-    return FLB_FILTER_MODIFIED;
+    return ret;
 }
 
 static int cb_throttle_exit(void *data, struct flb_config *config)
 {
+    void *thr_res;
     struct flb_filter_throttle_ctx *ctx = data;
 
+    int s = pthread_cancel(ctx->ticker_data.thr);
+    if (s != 0) {
+        flb_plg_error(ctx->ins, "Unable to cancel ticker. Leaking context to avoid memory corruption.");
+        return 1;
+    }
+
+    s = pthread_join(ctx->ticker_data.thr, &thr_res);
+    if (s != 0) {
+        flb_plg_error(ctx->ins, "Unable to join ticker. Leaking context to avoid memory corruption.");
+        return 1;
+    }
+
+    if (thr_res != PTHREAD_CANCELED) {
+        flb_plg_error(ctx->ins, "Thread joined but was not canceled which is impossible.");
+    }
+
     flb_free(ctx->hash->table);
     flb_free(ctx->hash);
     flb_free(ctx);
     return 0;
 }
 
+static struct flb_config_map config_map[] = {
+    // rate
+    // window
+    // print_status
+    // interval
+    {
+     FLB_CONFIG_MAP_DOUBLE, "rate", THROTTLE_DEFAULT_RATE,
+     0, FLB_TRUE, offsetof(struct flb_filter_throttle_ctx, max_rate),
+     "Set throttle rate"
+    },
+    {
+     FLB_CONFIG_MAP_INT, "window", THROTTLE_DEFAULT_WINDOW,
+     0, FLB_TRUE, offsetof(struct flb_filter_throttle_ctx, window_size),
+     "Set throttle window"
+    },
+    {
+     FLB_CONFIG_MAP_BOOL, "print_status", THROTTLE_DEFAULT_STATUS,
+     0, FLB_TRUE, offsetof(struct flb_filter_throttle_ctx, print_status),
+     "Set whether or not to print status information"
+    },
+    {
+     FLB_CONFIG_MAP_STR, "interval", THROTTLE_DEFAULT_INTERVAL,
+     0, FLB_TRUE, offsetof(struct flb_filter_throttle_ctx, slide_interval),
+     "Set the slide interval"
+    },
+    /* EOF */
+    {0}
+};
+
 struct flb_filter_plugin filter_throttle_plugin = {
     .name         = "throttle",
     .description  = "Throttle messages using sliding window algorithm",
     .cb_init      = cb_throttle_init,
     .cb_filter    = cb_throttle_filter,
     .cb_exit      = cb_throttle_exit,
+    .config_map   = config_map,
     .flags        = 0
 };
