@@ -2,8 +2,7 @@
 
 /*  Fluent Bit Throttling
  *  ==========
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
- *  Copyright (C) 2015-2018 Treasure Data Inc.
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -23,16 +22,22 @@
 
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_filter.h>
+#include <fluent-bit/flb_pthread.h>
 
 /* actions */
 #define THROTTLE_RET_KEEP  0
 #define THROTTLE_RET_DROP  1
 
 /* defaults */
-#define THROTTLE_DEFAULT_RATE  1
-#define THROTTLE_DEFAULT_WINDOW  5
+#define THROTTLE_DEFAULT_RATE "1"
+#define THROTTLE_DEFAULT_WINDOW  "5"
 #define THROTTLE_DEFAULT_INTERVAL  "1"
-#define THROTTLE_DEFAULT_STATUS FLB_FALSE;
+#define THROTTLE_DEFAULT_STATUS "false"
+
+struct ticker {
+    pthread_t thr;
+    double seconds;
+};
 
 struct flb_filter_throttle_ctx {
     double    max_rate;
@@ -43,12 +48,9 @@ struct flb_filter_throttle_ctx {
     /* internal */
     struct throttle_window *hash;
     struct flb_filter_instance *ins;
+    struct ticker ticker_data;
 };
 
-struct ticker {
-    struct flb_filter_throttle_ctx *ctx;
-    bool done;
-    double seconds;
-};
+
 
 #endif
