@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2015-2022 The Fluent Bit Authors
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -351,6 +351,10 @@ static int in_dummy_init(struct flb_input_instance *in,
 
     flb_input_set_context(in, ctx);
 
+    if (ctx->flush_on_startup) {
+        in_dummy_collect(in, config, ctx);
+    }
+
     ret = flb_input_set_collector_time(in,
                                        in_dummy_collect,
                                        tm.tv_sec,
@@ -444,6 +448,11 @@ static struct flb_config_map config_map[] = {
     0, FLB_TRUE, offsetof(struct flb_dummy, fixed_timestamp),
     "used a fixed timestamp, allows the message to pre-generated once."
    },
+   {
+    FLB_CONFIG_MAP_BOOL, "flush_on_startup", "false",
+    0, FLB_TRUE, offsetof(struct flb_dummy, flush_on_startup),
+    "generate the first event on startup"
+   },
    {0}
 };
 
