@@ -2,9 +2,8 @@
 
 /*  Serial input plugin for Fluent Bit
  *  ==================================
- *  Copyright (C) 2019-2021 The Fluent Bit Authors
+ *  Copyright (C) 2015-2024 The Fluent Bit Authors
  *  Copyright (C) 2015-2016 Takeshi HASEGAWA
- *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -29,6 +28,7 @@
 #include <msgpack.h>
 
 #include <fluent-bit/flb_pack.h>
+#include <fluent-bit/flb_log_event_encoder.h>
 
 struct flb_in_serial_config {
     int fd;           /* Socket to destination/backend */
@@ -39,15 +39,16 @@ struct flb_in_serial_config {
 
     /* config */
     int min_bytes;
-    const char *file;
-    const char *bitrate;
+    flb_sds_t file;
+    flb_sds_t bitrate;
 
     /* separator */
     int sep_len;
-    const char *separator;
+    flb_sds_t separator;
 
     /* Incoming format: JSON only for now */
     int format;
+    flb_sds_t format_str;
 
     struct termios tio;
     struct termios tio_orig;
@@ -61,6 +62,7 @@ struct flb_in_serial_config {
 
     /* Input instance reference */
     struct flb_input_instance *i_ins;
+    struct flb_log_event_encoder *log_encoder;
 
     /*
      * If (format == FLB_SERIAL_FORMAT_JSON), we use this pack_state
