@@ -28,6 +28,7 @@
 #include "flb_tests_runtime.h"
 
 #define JSON_CONTENT_TYPE "application/json"
+#define JSON_CHARSET_CONTENT_TYPE "application/json; charset=utf-8"
 
 struct http_client_ctx {
     struct flb_upstream      *u;
@@ -350,14 +351,92 @@ void flb_test_http_successful_response_code(char *response_code)
     test_ctx_destroy(ctx);
 }
 
+void flb_test_http_json_charset_header(char *response_code)
+{
+    struct flb_lib_out_cb cb_data;
+    struct test_ctx *ctx;
+    struct flb_http_client *c;
+    int ret;
+    int num;
+    size_t b_sent;
+
+    char *buf = "[{\"test\":\"msg\"}]";
+
+    clear_output_num();
+
+    cb_data.cb = cb_check_result_json;
+    cb_data.data = "\"test\":\"msg\"";
+
+    ctx = test_ctx_create(&cb_data);
+    if (!TEST_CHECK(ctx != NULL)) {
+        TEST_MSG("test_ctx_create failed");
+        exit(EXIT_FAILURE);
+    }
+
+    ret = flb_input_set(ctx->flb, ctx->i_ffd,
+                        "http2", "off",
+                        "successful_response_code", response_code,
+                         NULL);
+    TEST_CHECK(ret == 0);
+
+    ret = flb_output_set(ctx->flb, ctx->o_ffd,
+                         "match", "*",
+                         "format", "json",
+                         NULL);
+    TEST_CHECK(ret == 0);
+
+    /* Start the engine */
+    ret = flb_start(ctx->flb);
+    TEST_CHECK(ret == 0);
+
+    ctx->httpc = http_client_ctx_create();
+    TEST_CHECK(ctx->httpc != NULL);
+
+    flb_time_msleep(1500);
+
+    c = flb_http_client(ctx->httpc->u_conn, FLB_HTTP_POST, "/", buf, strlen(buf),
+                        "127.0.0.1", 9880, NULL, 0);
+    ret = flb_http_add_header(c, FLB_HTTP_HEADER_CONTENT_TYPE, strlen(FLB_HTTP_HEADER_CONTENT_TYPE),
+                              JSON_CHARSET_CONTENT_TYPE, strlen(JSON_CHARSET_CONTENT_TYPE));
+    TEST_CHECK(ret == 0);
+    if (!TEST_CHECK(c != NULL)) {
+        TEST_MSG("http_client failed");
+        exit(EXIT_FAILURE);
+    }
+
+    ret = flb_http_do(c, &b_sent);
+    if (!TEST_CHECK(ret == 0)) {
+        TEST_MSG("ret error. ret=%d\n", ret);
+    }
+    else if (!TEST_CHECK(b_sent > 0)){
+        TEST_MSG("b_sent size error. b_sent = %lu\n", b_sent);
+    }
+    else if (!TEST_CHECK(c->resp.status == atoi(response_code))) {
+        TEST_MSG("http response code error. expect: %d, got: %d\n", atoi(response_code), c->resp.status);
+    }
+
+    /* waiting to flush */
+    flb_time_msleep(1500);
+
+    num = get_output_num();
+    if (!TEST_CHECK(num > 0))  {
+        TEST_MSG("no outputs");
+    }
+    flb_http_client_destroy(c);
+    flb_upstream_conn_release(ctx->httpc->u_conn);
+    test_ctx_destroy(ctx);
+}
+
 void flb_test_http_successful_response_code_200()
 {
     flb_test_http_successful_response_code("200");    
+    flb_test_http_json_charset_header("200");
 }
 
 void flb_test_http_successful_response_code_204()
 {
     flb_test_http_successful_response_code("204");    
+    flb_test_http_json_charset_header("204");
 }
 
 void flb_test_http_failure_400_bad_json() {
@@ -468,7 +547,8 @@ void flb_test_http_failure_400_bad_disk_write()
 
     flb_time_msleep(5000);
 
-    ret = chmod("/tmp/http-input-test-404-bad-write", 000);
+    rename("/tmp/http-input-test-404-bad-write", 
+           "/tmp/http-input-test-404-bad-write.fail");
     TEST_CHECK(ret == 0);
 
     ctx->httpc = http_client_ctx_create();
@@ -495,11 +575,9 @@ void flb_test_http_failure_400_bad_disk_write()
         TEST_MSG("http response code error. expect: %d, got: %d\n", 400, c->resp.status);
     }
 
-    chmod("/tmp/http-input-test-404-bad-write/http.0", 0700);
-    rmdir("/tmp/http-input-test-404-bad-write/http.0");
-
-    chmod("/tmp/http-input-test-404-bad-write", 0700);
-    rmdir("/tmp/http-input-test-404-bad-write");
+    rename("/tmp/http-input-test-404-bad-write.fail", 
+           "/tmp/http-input-test-404-bad-write");
+    TEST_CHECK(ret == 0);
 
     /* waiting to flush */
     flb_time_msleep(1500);
